#include "Mreq.h"

Mreq *Mreq::getMClass(char *address, int numsrc) {

	int family = getAddrType(address);

	if (family == AF_INET) {
#ifdef SIOCSIPMSFILTER
		if (numsrc > 0)
			return new MreqV4Filter();
		else
#endif
			return new MreqV4();
	} 
#ifdef IPV6_JOIN_GROUP
	else if (family == AF_INET6) {
		return new MreqV6();
	}
#endif

	return NULL;

}

int Mreq::getAddrType(char *address) {

	if (strchr(address, ':') != NULL)
		return AF_INET6;

	return AF_INET;

}

int MreqV4::join(int sockfd) {

	return setsockopt(sockfd, IPPROTO_IP, IP_ADD_MEMBERSHIP, &mreq, sizeof(mreq));

}

int MreqV4::drop(int sockfd) {

	return setsockopt(sockfd, IPPROTO_IP, IP_DROP_MEMBERSHIP, &mreq, sizeof(mreq));

}

void MreqV4::setGroup(char *group, char *sources[], int numsrc, int fmode) {

	inet_pton(AF_INET, group, &mreq.imr_multiaddr);
	mreq.imr_interface.s_addr = INADDR_ANY;

}

char *MreqV4::getGroup() {

	char *group = (char *)malloc(sizeof(char)*16);

	inet_ntop(AF_INET, &mreq.imr_multiaddr, group, 16);

	return group;

}

#ifdef IPV6_JOIN_GROUP

int MreqV6::join(int sockfd) {

	return setsockopt(sockfd, IPPROTO_IPV6, IPV6_JOIN_GROUP, &mreq, sizeof(mreq));

}

int MreqV6::drop(int sockfd) {

	return setsockopt(sockfd, IPPROTO_IPV6, IPV6_LEAVE_GROUP, &mreq, sizeof(mreq));

}

void MreqV6::setGroup(char *group, char *sources[], int numsrc, int fmode) {

	inet_pton(AF_INET6, group, &mreq.ipv6mr_multiaddr);
	mreq.ipv6mr_interface = 0;

}

char *MreqV6::getGroup() {

	char *group = (char *)malloc(sizeof(char)*46);

	inet_ntop(AF_INET6, &mreq.ipv6mr_multiaddr, group, 46);

	return group;

}

#endif

#ifdef SIOCSIPMSFILTER

int MreqV4Filter::join(int sockfd) {

	return ioctl(sockfd, SIOCSIPMSFILTER, mreq);

}

int MreqV4Filter::drop(int sockfd) {

	return setsockopt(sockfd, IPPROTO_IP, IP_DROP_MEMBERSHIP, mreq, sizeof(*mreq));

}

void MreqV4Filter::setGroup(char *group, char *sources[], int numsrc, int fmode) {

	int i;

	mreq = (struct ip_msfilter *)malloc(IP_MSFILTER_SIZE(numsrc));

	inet_pton(AF_INET, group, &mreq->imsf_multiaddr);
	mreq->imsf_interface.s_addr = INADDR_ANY;
	mreq->imsf_fmode = fmode;
	mreq->imsf_numsrc = numsrc;

	for(i = 0; i < numsrc; i++)
		inet_pton(AF_INET, sources[i], &mreq->imsf_slist[i]);

}

char *MreqV4Filter::getGroup() {

	char *group = (char *)malloc(sizeof(char)*16);

	inet_ntop(AF_INET, &mreq->imsf_multiaddr, group, 16);

	return group;

}

#endif
